#!/usr/bin/env python3
"""Main functions.

Author: SADIK KUZU (c) 2021-2022
"""
from __future__ import annotations

import argparse
from math import log10
from math import sqrt

_directions = [["up", "right"], ["down", "left"]]


def to_where(A: int) -> str:
    """Get the number in the squiral and return back to next direction as a return value.

    Args:
        A (int): the number in the squiral

    Returns:
        str: next direction: "up", "right", "down", "left"
    """
    if A == 1:
        return "right"
    initial = int(sqrt(A))
    initial2 = initial**2
    if A == initial2:
        return to_where(A - 1)
    middle = initial * (initial + 1)
    first = initial % 2
    second = 0 if A <= middle else 1
    return _directions[first][second]


def next_point(row: int, col: int, direction: str) -> tuple:
    """Return next point indices according to current indices and direction.

    Args:
        row (int): row index
        col (int): column index
        direction (str): next direction: "up", "right", "down", "left"

    Returns:
        tuple: next point indices
    """
    if direction == "right":
        col += 1
    elif direction == "left":
        col -= 1
    elif direction == "up":
        row -= 1
    elif direction == "down":
        row += 1
    return (row, col)


def produce(size: int) -> list:
    """Construct double array wrt square size.

    Args:
        size (int): square size

    Returns:
        list: squiral numbers in 2D array
    """
    if size < 1:
        return []
    s = [[0 for i in range(size)] for j in range(size)]
    r = c = (size - 1) // 2
    A = 1
    s[r][c] = A
    while size > 1:
        r, c = next_point(r, c, to_where(A))
        A += 1
        s[r][c] = A
        if A == size**2:
            return s
    return s


def printout(s: list):
    """Printout 2D array.

    Args:
        s (list): squiral numbers in 2D array
    """
    if s:
        R = C = len(s)
        RJ = int(log10(R * C)) + 1
        for r in range(R):
            for c in range(C):
                print(str(s[r][c]).rjust(RJ), end=" ")
            print()


if __name__ == "__main__":  # pragma: no cover
    parser = argparse.ArgumentParser()
    parser.add_argument("size", help="squiral size")
    args = parser.parse_args()

    print("Welcome to Squiral!")
    print("Here is an example:")
    try:
        size = int(args.size)
    except Exception:
        size = 5
    printout(produce(size))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['squiral']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['squiral = squiral.main:main_cli']}

setup_kwargs = {
    'name': 'squiral',
    'version': '0.3.1',
    'description': 'squiral - square spiral',
    'long_description': "# squiral\n[![](https://img.shields.io/pypi/v/squiral)](https://pypi.org/project/squiral/)\n[![made-with-python](https://img.shields.io/badge/Made%20with-Python-1f425f.svg)](https://www.python.org/)\n[![](https://img.shields.io/pypi/pyversions/squiral.svg)](https://pypi.org/project/squiral/)\n[![Downloads](https://pepy.tech/badge/squiral)](https://pepy.tech/project/squiral)\n<br/>\n[![pre-commit.ci status](https://results.pre-commit.ci/badge/github/sadikkuzu/squiral/main.svg)](https://results.pre-commit.ci/latest/github/sadikkuzu/squiral/main)\n[![Python lint&test](https://github.com/sadikkuzu/squiral/actions/workflows/python-package.yml/badge.svg)](https://github.com/sadikkuzu/squiral/actions/workflows/python-package.yml)\n[![Publish Python Package](https://github.com/sadikkuzu/squiral/actions/workflows/python-publish.yml/badge.svg)](https://github.com/sadikkuzu/squiral/actions/workflows/python-publish.yml)\n\n**squ**are sp**iral**\n\n```\nWelcome to Squiral!\nHere is an example:\n21 22 23 24 25\n20  7  8  9 10\n19  6  1  2 11\n18  5  4  3 12\n17 16 15 14 13\n```\n\n**squiral** is a simple and lightweight Python library for generating square spiral patterns.<br/>\nWith **squiral**, you can easily create a wide range of square spiral designs, from simple to complex.<br/>\n\n**squiral** is easy to use and integrate into existing projects,<br/>\nmaking it a great tool for data visualization, computer graphics, and other applications.<br/>\nWith **squiral**, you can quickly and easily generate square spiral patterns<br/>\nthat are both beautiful and functional.<br/>\nWhether you're working on a personal project or a professional one,<br/>\n**squiral**'s square spiral generator is the perfect tool<br/>\nfor adding unique and visually striking designs to your work.\n\nThe basic idea behind printing this matrix is<br/>\nto start from the middle of the matrix and then moving:<br/>\n`right` >> `down` >> `left` >> `up`<br/>\nand not returning to the same row again.\n\n### Install\n\n#### [PyPI](https://pypi.org/project/squiral/)\n\n```shell\npip install squiral\n```\n\n#### [GitHub](https://github.com/sadikkuzu/squiral)\n\n```shell\npip install git+https://github.com/sadikkuzu/squiral.git\n```\n\n### Usage\n\n#### In console:\n\n```console\nbuddha@dharma:~$ squiral 7\n43 44 45 46 47 48 49\n42 21 22 23 24 25 26\n41 20  7  8  9 10 27\n40 19  6  1  2 11 28\n39 18  5  4  3 12 29\n38 17 16 15 14 13 30\n37 36 35 34 33 32 31\nbuddha@dharma:~$ squiral --help\nusage: squiral [-h] size\n\npositional arguments:\n  size        squiral size\n\noptional arguments:\n  -h, --help  show this help message and exit\n```\n\n#### In python:\n\n```python\n>>> import squiral as sq\n>>> sq.printout(sq.produce(5))\n21 22 23 24 25\n20  7  8  9 10\n19  6  1  2 11\n18  5  4  3 12\n17 16 15 14 13\n>>>\n```\n",
    'author': 'SADIK KUZU',
    'author_email': 'sadikkuzu@hotmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/sadikkuzu/squiral',
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.8.1,<4.0.0',
}


setup(**setup_kwargs)
